/*
 * Aipo is a groupware program developed by Aimluck,Inc.
 * Copyright (C) 2004-2011 Aimluck,Inc.
 * http://www.aipo.com
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.aimluck.eip.account.util;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

import org.apache.cayenne.exp.Expression;
import org.apache.cayenne.exp.ExpressionFactory;
import org.apache.jetspeed.om.security.Group;
import org.apache.jetspeed.om.security.JetspeedUser;
import org.apache.jetspeed.om.security.Role;
import org.apache.jetspeed.om.security.UserNamePrincipal;
import org.apache.jetspeed.services.JetspeedSecurity;
import org.apache.jetspeed.services.PsmlManager;
import org.apache.jetspeed.services.logging.JetspeedLogFactoryService;
import org.apache.jetspeed.services.logging.JetspeedLogger;
import org.apache.jetspeed.services.security.JetspeedSecurityException;
import org.apache.jetspeed.services.security.UnknownUserException;
import org.apache.turbine.util.RunData;
import org.apache.velocity.context.Context;

import com.aimluck.commons.utils.ALStringUtil;
import com.aimluck.eip.cayenne.om.account.AvzMGroupsend;
import com.aimluck.eip.cayenne.om.account.AvzMRole;
import com.aimluck.eip.cayenne.om.account.AvzMUserGroupsend;
import com.aimluck.eip.cayenne.om.account.AvzMUserRole;
import com.aimluck.eip.cayenne.om.account.EipMCompany;
import com.aimluck.eip.cayenne.om.account.EipMPosition;
import com.aimluck.eip.cayenne.om.account.EipMPost;
import com.aimluck.eip.cayenne.om.account.EipMUserPosition;
import com.aimluck.eip.cayenne.om.portlet.AvzTMailSend;
import com.aimluck.eip.cayenne.om.portlet.AvzTMailSendRecvAcl;
import com.aimluck.eip.cayenne.om.portlet.AvzTScheduleAcl;
import com.aimluck.eip.cayenne.om.portlet.EipMAddressGroup;
import com.aimluck.eip.cayenne.om.portlet.EipMAddressbook;
import com.aimluck.eip.cayenne.om.portlet.EipMMailAccount;
import com.aimluck.eip.cayenne.om.portlet.EipTBlog;
import com.aimluck.eip.cayenne.om.portlet.EipTBlogFootmarkMap;
import com.aimluck.eip.cayenne.om.portlet.EipTMail;
import com.aimluck.eip.cayenne.om.portlet.EipTMailFolder;
import com.aimluck.eip.cayenne.om.portlet.EipTTodo;
import com.aimluck.eip.cayenne.om.portlet.EipTTodoCategory;
import com.aimluck.eip.cayenne.om.portlet.EipTWorkflowRequestMap;
import com.aimluck.eip.cayenne.om.security.TurbineGroup;
import com.aimluck.eip.cayenne.om.security.TurbineUser;
import com.aimluck.eip.cayenne.om.security.TurbineUserGroupRole;
import com.aimluck.eip.common.ALBaseUser;
import com.aimluck.eip.common.ALEipConstants;
import com.aimluck.eip.mail.ALMailFactoryService;
import com.aimluck.eip.mail.ALMailHandler;
import com.aimluck.eip.orm.Database;
import com.aimluck.eip.orm.query.Operations;
import com.aimluck.eip.orm.query.ResultList;
import com.aimluck.eip.orm.query.SelectQuery;
import com.aimluck.eip.services.config.ALConfigService;
import com.aimluck.eip.services.config.ALConfigHandler.Property;
import com.aimluck.eip.services.social.ALApplicationService;
import com.aimluck.eip.user.beans.UserGroupLiteBean;
import com.aimluck.eip.util.ALEipUtils;

/**
 * ユーザーアカウントのユーティリティクラスです
 */
public class AccountUtils {

  /** logger */
  private static final JetspeedLogger logger = JetspeedLogFactoryService.getLogger(AccountUtils.class.getName());

  /** CSVファイル名 */
  public static final String FOLDER_TMP_FOR_USERINFO_CSV_FILENAME = "user_info.csv";

  public static final int CSV_FILE_COL_COUNT = 11;

  /** ユーザー名として使用可能な記号群 */
  public static final String[] USER_NAME_SYMBOLS = { ".", "-", "_" };

  // add by motegi start
  // グループ送信カテゴリ（部署グループ）
  // public static final String GROUPSEND_CATEGORY_POST = "P";

  public static final String GROUPSEND_CATEGORY_POST_NAME = "部署グループ";

  // add end

  /**
   * セッション中のエンティティIDで示されるユーザ情報を取得する。 論理削除されたユーザを取得した場合はnullを返す。
   * 
   * @param rundata
   * @param context
   * @return
   */
  public static ALBaseUser getBaseUser(RunData rundata, Context context) {
    String userid = ALEipUtils.getTemp(rundata, context, ALEipConstants.ENTITY_ID);
    try {
      if (userid == null) {
        logger.debug("Empty ID...");
        return null;
      }
      ALBaseUser user = (ALBaseUser) JetspeedSecurity.getUser(userid);
      // 削除済みユーザの取得は行わない。
      // By Haruo Kaneko
      if ("T".equals(user.getDisabled())) {
        return null;
      } else {
        return (ALBaseUser) JetspeedSecurity.getUser(userid);
      }
    } catch (UnknownUserException uex) {
      logger.error("UnknownUserException : UserID = " + userid);
      return null;
    } catch (Exception ex) {
      logger.error("Exception", ex);
      return null;
    }
  }

  /**
   * 
   * @param rundata
   * @param context
   * @return
   */
  public static EipMCompany getEipMCompany(RunData rundata, Context context) {
    EipMCompany result = null;
    String id = ALEipUtils.getTemp(rundata, context, ALEipConstants.ENTITY_ID);
    try {
      if (id == null || Integer.valueOf(id) == null) {
        logger.debug("Empty ID...");
        return result;
      }

      Expression exp = ExpressionFactory.matchDbExp(EipMCompany.COMPANY_ID_PK_COLUMN, Integer.valueOf(id));
      List<EipMCompany> list = Database.query(EipMCompany.class, exp).fetchList();
      if (list == null || list.size() == 0) {
        logger.debug("Not found ID...");
        return result;
      }
      result = list.get(0);
    } catch (Exception ex) {
      logger.error("Exception", ex);
    }
    return result;
  }

  /**
   * セッションに格納されているIDを用いて、部署情報を取得します。
   * 
   * @param rundata
   * @param context
   * @return
   * @throws Exception
   */
  public static EipMPost getEipMPost(RunData rundata, Context context) throws Exception {
    EipMPost result = null;
    String id = ALEipUtils.getTemp(rundata, context, ALEipConstants.ENTITY_ID);
    try {
      if (id == null || Integer.valueOf(id) == null) {
        logger.debug("Empty ID...");
        // change start
        // return result;
        throw new Exception("エンティティIDが渡されていません。");
        // change end
      }

      Expression exp = ExpressionFactory.matchDbExp(EipMPost.POST_ID_PK_COLUMN, Integer.valueOf(id));
      List<EipMPost> list = Database.query(EipMPost.class, exp).fetchList();
      if (list == null || list.size() == 0) {
        logger.debug("Not found ID...");
        // change start
        // return result;
        throw new Exception("指定された部署情報がデータベースにありません。ID=" + id);
        // change end
      }
      result = list.get(0);
    } catch (Exception ex) {
      // change start
      // logger.error("Exception", ex);
      throw ex;
      // change end
    }
    return result;
  }

  /**
   * 
   * @param rundata
   * @param context
   * @return
   * @throws Exception
   */
  public static EipMPosition getEipMPosition(RunData rundata, Context context) throws Exception {
    EipMPosition result = null;
    String id = ALEipUtils.getTemp(rundata, context, ALEipConstants.ENTITY_ID);
    try {
      if (id == null || Integer.valueOf(id) == null) {
        logger.debug("Empty ID...");
        // change start
        // return result;
        throw new Exception("エンティティIDが渡されていません。");
        // change end
      }

      Expression exp = ExpressionFactory.matchDbExp(EipMPosition.POSITION_ID_PK_COLUMN, Integer.valueOf(id));
      List<EipMPosition> list = Database.query(EipMPosition.class, exp).fetchList();
      if (list == null || list.size() == 0) {
        logger.debug("Not found ID...");
        // change start
        // return result;
        throw new Exception("指定された役職情報がデータベースにありません。ID=" + id);
        // change end
      }
      result = list.get(0);
    } catch (Exception ex) {
      // change start
      // logger.error("Exception", ex);
      throw ex;
      // change end
    }
    return result;
  }

  public static String[] getCsvSplitStrings(String line) {
    if (line == null || line.equals("")) {
      return null;
    }

    try {
      List<String> list = new ArrayList<String>();
      int count_comma = 0;
      char c;
      StringBuffer token = new StringBuffer("");
      int len = line.length();
      for (int i = 0; i < len; i++) {
        c = line.charAt(i);
        if (c != ',' && i == len - 1) {
          token.append(c);
          list.add(token.toString());
        } else if (c == ',') {
          list.add(token.toString());
          token = new StringBuffer("");
          count_comma++;
          continue;
        } else {
          token.append(c);
        }
        if (count_comma > AccountUtils.CSV_FILE_COL_COUNT) {
          break;
        }
      }

      if (line.endsWith(",")) {
        list.add("");
      }

      String[] strings = new String[list.size()];
      strings = list.toArray(strings);
      return strings;
    } catch (Exception e) {
      return null;
    }
  }

  /**
   * ユーザーの所属する部署の一覧を取得します。
   * 
   * @param uid
   *            ユーザーID
   * @return 所属する部署リスト
   */
  public static List<UserGroupLiteBean> getPostBeanList(int uid) {
    SelectQuery<TurbineUserGroupRole> query = Database.query(TurbineUserGroupRole.class);
    Expression exp1 = ExpressionFactory.matchExp(TurbineUserGroupRole.TURBINE_USER_PROPERTY, Integer.valueOf(uid));
    Expression exp2 = ExpressionFactory.greaterExp(TurbineUserGroupRole.TURBINE_GROUP_PROPERTY, Integer.valueOf(3));
    Expression exp3 = ExpressionFactory.noMatchExp(TurbineUserGroupRole.TURBINE_GROUP_PROPERTY + "." + TurbineGroup.OWNER_ID_PROPERTY, null);
    query.setQualifier(exp1);
    query.andQualifier(exp2);
    query.andQualifier(exp3);
    List<TurbineUserGroupRole> list = query.fetchList();

    if (list == null || list.size() < 0) {
      return null;
    }

    List<UserGroupLiteBean> resultList = new ArrayList<UserGroupLiteBean>();

    TurbineGroup group = null;
    UserGroupLiteBean bean = null;
    for (TurbineUserGroupRole ugr : list) {
      group = ugr.getTurbineGroup();
      bean = new UserGroupLiteBean();
      bean.initField();
      bean.setGroupId(group.getName());
      bean.setName(group.getGroupAliasName());
      resultList.add(bean);
    }

    return resultList;
  }

  /**
   * 指定した ID のユーザが削除済みかどうかを調べる。
   * 
   * @param userId
   * @return
   */
  public static boolean getUserIsDisabledOrDeleted(String userId) {
    if (userId == null || userId.equals("")) {
      return true;
    }
    String disabled;
    try {
      SelectQuery<TurbineUser> query = Database.query(TurbineUser.class);
      Expression exp = ExpressionFactory.matchDbExp(TurbineUser.USER_ID_PK_COLUMN, Integer.valueOf(userId));
      query.setQualifier(exp);
      List<TurbineUser> destUserList = query.fetchList();
      if (destUserList == null || destUserList.size() <= 0) {
        return true;
      }
      disabled = (destUserList.get(0)).getDisabled();
      return ("T".equals(disabled) || "N".equals(disabled));
    } catch (Exception ex) {
      logger.error("Exception", ex);
      return true;
    }
  }

  /**
   * 指定されたユーザーが削除／無効化されたとき、申請が来ているワークフローを全て承認します。
   * 
   * @param uid
   */
  public static boolean acceptWorkflow(int uid) {
    try {
      String userId = Integer.toString(uid);

      // 申請が来ているワークフローを取得する
      SelectQuery<EipTWorkflowRequestMap> workflow_request_map_query = Database.query(EipTWorkflowRequestMap.class);
      Expression workflow_exp = ExpressionFactory.matchExp(EipTWorkflowRequestMap.USER_ID_PROPERTY, userId);
      Expression workflow_exp2 = ExpressionFactory.matchExp(EipTWorkflowRequestMap.STATUS_PROPERTY, "C");
      workflow_request_map_query.setQualifier(workflow_exp.andExp(workflow_exp2));
      List<EipTWorkflowRequestMap> workflow_request_map_list = workflow_request_map_query.fetchList();
      EipTWorkflowRequestMap workflow_request_map = null;
      int list_size = workflow_request_map_list.size();

      // 申請が来ているワークフローの数だけ繰り返す
      for (int j = 0; j < list_size; j++) {
        workflow_request_map = workflow_request_map_list.get(j);

        // ワークフローを最後の人まで見ていく
        int request_number = workflow_request_map.getOrderIndex();
        while (true) {
          // 次の人がいるかどうか
          SelectQuery<EipTWorkflowRequestMap> workflow_request_map_query2 = Database.query(EipTWorkflowRequestMap.class);
          Expression workflow_exp3 =
            ExpressionFactory.matchExp(EipTWorkflowRequestMap.EIP_TWORKFLOW_REQUEST_PROPERTY, workflow_request_map.getEipTWorkflowRequest());
          Expression workflow_exp4 = ExpressionFactory.matchExp(EipTWorkflowRequestMap.ORDER_INDEX_PROPERTY, Integer.valueOf(request_number + 1));
          workflow_request_map_query2.setQualifier(workflow_exp3.andExp(workflow_exp4));
          List<EipTWorkflowRequestMap> workflow_request_map_list2 = workflow_request_map_query2.fetchList();

          // 自身を自動承認状態にする
          workflow_request_map.setStatus("T");
          if (workflow_request_map_list2.size() == 1) {
            // 次の人が見つかった
            EipTWorkflowRequestMap workflow_request_map2 = workflow_request_map_list2.get(0);
            if (getUserIsDisabledOrDeleted(workflow_request_map2.getUserId().toString())) {
              // 次の人が削除済み、もしくは無効化されていたら自動承認した上で次の人に回す
              workflow_request_map2.setStatus("T");
              request_number += 1;
            } else {
              // 次の人を確認状態にして終了
              workflow_request_map2.setStatus("C");
              break;
            }
          } else {
            // 次の人が見つからなければ、最後まで行ったことになるので終了
            workflow_request_map.getEipTWorkflowRequest().setProgress("A");
            break;
          }
        }
      }
      return true;
    } catch (Exception e) {
      logger.error("Exception", e);
      return false;
    }
  }

  /**
   * 管理者権限を持ったユーザを一人、管理者権限剥奪・無効化・削除しても<br/> 最低限必要な管理者権限を持ったユーザ数を割らないかどうかを返します。
   * 
   * @return
   */
  public static boolean isAdminDeletable() {
    return isAdminDeletable(1);
  }

  /**
   * 管理者権限を持ったユーザを指定人数、管理者権限剥奪・無効化・削除しても<br/>
   * 最低限必要な管理者権限を持ったユーザ数を割らないかどうかを返します。
   * 
   * @param admin_count
   * @return
   */
  public static boolean isAdminDeletable(int admin_count) {
    try {
      int minimum_admin = Integer.valueOf(ALConfigService.get(Property.MINIMUM_ADMINISTRATOR_USER_COUNT));
      Group group = JetspeedSecurity.getGroup("LoginUser");
      Role adminrole = JetspeedSecurity.getRole("admin");
      int current_admin_count =
        Database.query(TurbineUserGroupRole.class).where(
          Operations.eq(TurbineUserGroupRole.TURBINE_ROLE_PROPERTY, adminrole.getId()),
          Operations.eq(TurbineUserGroupRole.TURBINE_GROUP_PROPERTY, group.getId()),
          Operations.ne(TurbineUserGroupRole.TURBINE_USER_PROPERTY, 1),
          Operations.eq(TurbineUserGroupRole.TURBINE_USER_PROPERTY + "." + TurbineUser.DISABLED_PROPERTY, "F")).distinct(true).getCount();
      int admin_count_will = current_admin_count - admin_count;
      if (admin_count_will < 0) {
        admin_count_will = 0;
      }
      return minimum_admin <= admin_count_will;
    } catch (JetspeedSecurityException e) {
      logger.error("Exception", e);
      return false;
    }
  }

  /**
   * 与えられたユーザー名に使われている記号が、使用できるものかを確認します。
   * 
   * @return
   */
  public static boolean isValidSymbolUserName(String name) {
    List<String> symbols = Arrays.asList(USER_NAME_SYMBOLS);
    for (char c : name.toCharArray()) {
      if (ALStringUtil.isSymbol(c) && !symbols.contains(String.valueOf(c))) {
        return false;
      }
    }
    return true;
  }

  // add by motegi start
  /**
   * 役割オブジェクトモデルを取得します。 <BR>
   * 
   * @param rundata
   *            JetSpeedランデータ
   * @param context
   *            JetSpeedコンテキスト
   * @return 役割オブジェクト
   * @throws Exception
   */
  public static AvzMRole getAvzMRole(RunData rundata, Context context) throws Exception {
    String entity_id = ALEipUtils.getTemp(rundata, context, ALEipConstants.ENTITY_ID);
    try {
      if (entity_id == null || Integer.valueOf(entity_id) == null) {
        // $EntityClass$ IDが空の場合
        logger.debug("[$EntityClass$] Empty ID...");
        throw new Exception("エンティティIDが渡されていません。");
      }

      SelectQuery<AvzMRole> query = Database.query(AvzMRole.class);
      Expression exp = ExpressionFactory.matchDbExp(AvzMRole.ROLE_ID_PK_COLUMN, entity_id);
      // ExpressionFactory.matchDbExp("role_id", entity_id);
      query.setQualifier(exp);
      ResultList<AvzMRole> list = query.getResultList();

      if (list == null || list.size() == 0) {
        // 指定したAvzMRole IDのレコードが見つからない場合
        logger.debug("[AvzMRole] Not found ID...");
        throw new Exception("指定された役割がデータベースにありません。ID=" + entity_id);
      }

      return list.get(0);

    } catch (Exception ex) {
      throw ex;
    }
  }

  /**
   * グループ送信オブジェクトモデルを取得します。 <BR>
   * 
   * @param rundata
   *            JetSpeedランデータ
   * @param context
   *            JetSpeedコンテキスト
   * @return グループ送信オブジェクト
   * @throws Exception
   */
  public static AvzMGroupsend getAvzMGroupsend(RunData rundata, Context context) throws Exception {
    String entity_id = ALEipUtils.getTemp(rundata, context, ALEipConstants.ENTITY_ID);
    try {
      if (entity_id == null || Integer.valueOf(entity_id) == null) {
        // $EntityClass$ IDが空の場合
        logger.debug("[$EntityClass$] Empty ID...");
        throw new Exception("エンティティIDが渡されていません。");
      }

      SelectQuery<AvzMGroupsend> query = Database.query(AvzMGroupsend.class);
      Expression exp = ExpressionFactory.matchDbExp(AvzMGroupsend.GROUPSEND_ID_PK_COLUMN, entity_id);
      query.setQualifier(exp);
      ResultList<AvzMGroupsend> list = query.getResultList();

      if (list == null || list.size() == 0) {
        // 指定したAvzMRole IDのレコードが見つからない場合
        logger.debug("[AvzMOccupationalCategory] Not found ID...");
        throw new Exception("指定されたグループ送信がデータベースにありません。ID=" + entity_id);
      }

      return list.get(0);

    } catch (Exception ex) {
      throw ex;
    }
  }

  /**
   * メールアカウント削除
   * 
   * @param accountId
   *            アカウントID
   * @param msgList
   *            エラーメッセージリスト
   * @return
   * @throws Exception
   */
  public static void deleteAccount(int accountId, List<String> msgList) throws Exception {

    // 代理送受信制御を削除する。
    String deleteAvzTMailSendRecvAclSql = "DELETE FROM AVZ_T_MAIL_SEND_RECV_ACL WHERE ACCOUNT_ID = #bind($accountId 'INTEGER')";
    Database.sql(AvzTMailSendRecvAcl.class, deleteAvzTMailSendRecvAclSql).param("accountId", accountId).execute();

    // Web受信メールを削除する。
    String deleteEipTMailSql = "DELETE FROM EIP_T_MAIL WHERE ACCOUNT_ID = #bind($accountId 'INTEGER')";
    Database.sql(EipTMail.class, deleteEipTMailSql).param("accountId", accountId).execute();

    // Web送信メールを削除する。
    String deleteAvzTMailSendSql = "DELETE FROM AVZ_T_MAIL_SEND WHERE ACCOUNT_ID = #bind($accountId 'INTEGER')";
    Database.sql(AvzTMailSend.class, deleteAvzTMailSendSql).param("accountId", accountId).execute();

    // メールフォルダを削除する。
    String deleteEipTMailFolderSql = "DELETE FROM EIP_T_MAIL_FOLDER WHERE ACCOUNT_ID = #bind($accountId 'INTEGER')";
    Database.sql(EipTMailFolder.class, deleteEipTMailFolderSql).param("accountId", accountId).execute();

    EipMMailAccount account = Database.get(EipMMailAccount.class, accountId);
    if (account == null) {
      throw new Exception("指定されたメールアカウントがデータベースにありません。ID=" + accountId);
    }

    // メールファイル格納ディレクトリを削除する。
    String orgId = Database.getDomainName();
    ALMailHandler handler = ALMailFactoryService.getInstance().getMailHandler();
    handler.removeAccount(orgId, account.getUserId(), accountId);

    Database.delete(account);

    return;
  }

  /**
   * 指定ユーザーのWebメールアカウントを取得する。
   * 
   * @param user_id
   *            ユーザーID
   * @return Webメールアカウントリスト
   * @author motegi
   */
  public static List<EipMMailAccount> getEipMMailAccounts(int user_id) {

    SelectQuery<EipMMailAccount> query = Database.query(EipMMailAccount.class);
    query.select(EipMMailAccount.ACCOUNT_ID_PK_COLUMN);
    query.select(EipMMailAccount.ACCOUNT_NAME_COLUMN);
    query.select("MAIL_ADDRESS");
    Expression exp = ExpressionFactory.matchExp(EipMMailAccount.USER_ID_PROPERTY, Integer.valueOf(user_id));

    List<EipMMailAccount> dbList = query.setQualifier(exp).fetchList();
    return dbList;
  }

  /**
   * 指定ユーザーの削除を行なう
   * 
   * @param user
   *            削除対象ユーザー
   * @param msgList
   *            エラーメッセージリスト
   * @return 処理結果（正常:true、異常:false）
   * @throws Exception
   */
  public static boolean deleteUser(TurbineUser user, List<String> msgList) throws Exception {

    user.setPositionId(Integer.valueOf(0));
    user.setDisabled("T");

    String userId = user.getUserId().toString();

    // 対象ユーザのユーザーグループロールをすべて削除する
    SelectQuery<TurbineUserGroupRole> query2 = Database.query(TurbineUserGroupRole.class);
    Expression exp2 = ExpressionFactory.matchExp(TurbineUserGroupRole.TURBINE_USER_PROPERTY, userId);
    query2.setQualifier(exp2);
    List<TurbineUserGroupRole> list4 = query2.fetchList();

    TurbineUserGroupRole ugr = null;
    for (int i = 0; i < list4.size(); i++) {
      ugr = list4.get(i);
      Database.delete(ugr);
    }

    // add by motegi start
    // スケジュール制御
    String deleteAvzTScheduleAcl = "DELETE FROM avz_t_schedule_acl WHERE USER_ID = #bind($userId 'INTEGER')";
    Database.sql(AvzTScheduleAcl.class, deleteAvzTScheduleAcl).param("userId", userId).execute();

    // ユーザーグループ送信削除
    String deleteAvzMUserGroupsend = "DELETE FROM avz_m_user_groupsend WHERE USER_ID = #bind($userId 'INTEGER')";
    Database.sql(AvzMUserGroupsend.class, deleteAvzMUserGroupsend).param("userId", userId).execute();

    // ユーザー役割削除
    String deleteAvzMUserRole = "DELETE FROM avz_m_user_role WHERE USER_ID = #bind($userId 'INTEGER')";
    Database.sql(AvzMUserRole.class, deleteAvzMUserRole).param("userId", userId).execute();

    // メールアカウント削除
    List<EipMMailAccount> accountList = AccountUtils.getEipMMailAccounts(Integer.parseInt(userId));
    if (accountList != null && accountList.size() > 0) {
      for (EipMMailAccount obj : accountList) {
        AccountUtils.deleteAccount(obj.getAccountId(), msgList);
      }
    }

    // 個人アドレス帳削除
    List<EipMAddressbook> addressList = AccountUtils.getEipMAddressbooks(Integer.parseInt(userId));
    if (addressList != null && addressList.size() > 0) {
      for (EipMAddressbook obj : addressList) {
        String deleteEipMAddressGroup = "DELETE FROM eip_t_addressbook_group_map WHERE ADDRESS_ID = #bind($addressId 'INTEGER')";
        Database.sql(EipMAddressGroup.class, deleteEipMAddressGroup).param("addressId", obj.getAddressId()).execute();
      }
    }

    String deleteEipMAddressGroup = "DELETE FROM eip_m_address_group WHERE OWNER_ID = #bind($userId 'INTEGER')";
    Database.sql(EipMAddressGroup.class, deleteEipMAddressGroup).param("userId", userId).execute();

    String deleteEipMAddressbook = "DELETE FROM eip_m_addressbook WHERE OWNER_ID = #bind($userId 'INTEGER')";
    Database.sql(EipMAddressbook.class, deleteEipMAddressbook).param("userId", userId).execute();

    // ユーザー表示順位削除
    String deleteEipMUserPosition = "DELETE FROM eip_m_user_position WHERE USER_ID = #bind($userId 'INTEGER')";
    Database.sql(EipMUserPosition.class, deleteEipMUserPosition).param("userId", userId).execute();
    // add end

    // ToDoを削除する
    String sql4 = "DELETE FROM eip_t_todo WHERE USER_ID = #bind($userId 'INTEGER')";
    Database.sql(EipTTodo.class, sql4).param("userId", userId).execute();

    String sql5 = "DELETE FROM eip_t_todo_category WHERE USER_ID = #bind($userId 'INTEGER')";
    Database.sql(EipTTodoCategory.class, sql5).param("userId", userId).execute();

    // ブログを削除する
    String sql6 = "DELETE FROM eip_t_blog WHERE OWNER_ID = #bind($userId 'INTEGER')";
    Database.sql(EipTBlog.class, sql6).param("userId", userId).execute();

    // ブログの足跡を削除する
    String sql7 = "DELETE FROM eip_t_blog_footmark_map WHERE USER_ID = #bind($userId 'INTEGER')";
    Database.sql(EipTBlogFootmarkMap.class, sql7).param("userId", userId).execute();

    // ワークフロー自動承認
    AccountUtils.acceptWorkflow(Integer.parseInt(userId));

    // ソーシャルアプリ関連データ削除
    ALApplicationService.deleteUserData(user.getLoginName());

    // PSMLを削除
    JetspeedUser juser = JetspeedSecurity.getUser(new UserNamePrincipal(user.getLoginName()));
    PsmlManager.removeUserDocuments(juser);

    // ユーザー名の先頭に"dummy_userid_"を追加
    String dummy_user_name = ALEipUtils.dummy_user_head + userId + "_" + user.getLoginName();
    user.setLoginName(dummy_user_name);
    user.setModified(new Date());

    return true;

  }

  /**
   * 指定したユーザーが登録した個人アドレス情報を取得する。
   * 
   * @param user_id
   *            ユーザーID
   * @return 個人アドレス情報リスト
   */
  public static List<EipMAddressbook> getEipMAddressbooks(int user_id) {
    SelectQuery<EipMAddressbook> query = Database.query(EipMAddressbook.class);
    Expression exp = ExpressionFactory.matchExp(EipMAddressbook.OWNER_ID_PROPERTY, user_id);
    query.andQualifier(exp);

    List<EipMAddressbook> addresses = query.fetchList();
    return addresses;
  }

  // add start 運用フェーズ課題・障害台帳No.１５２
  /**
   * MD5ハッシュ化処理
   * 
   * @param key
   *            ソース文字列
   * @return ハッシュ化済み文字列
   * @throws NoSuchAlgorithmException
   */
  public static String getMD5Digest(String key) throws NoSuchAlgorithmException {
    String strKey = key;
    MessageDigest md = MessageDigest.getInstance("MD5");
    md.update(strKey.getBytes());
    byte byDig[] = md.digest();
    String strOut = "";
    for (int i = 0; i < byDig.length; i++) {
      int d = byDig[i];
      if (d < 0) { // byte型では128～255が負値になっているので補正
        d += 256;
      }
      if (d < 16) { // 0～15は16進数で1けたになるので、2けたになるよう頭に0を追加
        strOut += "0";
      }
      strOut += Integer.toString(d, 16); // ダイジェスト値の1バイトを16進数2けたで表示
    }

    return strOut;
  }
  // add end
}
